<?php
/**
 * @file
 * Allows the site to send and receive user contacts to and from Text Marketer.
 */

namespace Drupal\textmarketer_contacts\Controller;

/**
 * Class Request.
 *
 * @package Drupal\textmarketer_contacts\Controller
 */
class Request {
  private $username;
  private $password;
  private $group_id;
  private $group_name;
  private $base_url;

  public function __construct($username = NULL, $password = NULL, $group_id = NULL, $group_name = NULL) {

    $this->username = is_null($username) ?
      variable_get('textmarketer_contacts_user_name', '') : $username;
    $this->password = is_null($password) ?
      variable_get('textmarketer_contacts_password', '') : $password;
    $this->group_id = is_null($group_id) ?
      variable_get('textmarketer_contacts_group_id', '') : $group_id;
    $this->group_name = is_null($group_name) ?
      variable_get('textmarketer_contacts_group_name', '') : $group_name;
    $this->base_url
      = "https://$this->username:$this->password@api.textmarketer.co.uk";
  }

  /**
   * Catches the telephone numbers on user registration and account update.
   *
   * @param array $edit
   *   An array containing the user data.
   */
  public function catchNumbers($edit) {

    $field_subscribe = variable_get('textmarketer_contacts_subscribe', '');
    $user_wrapper = new UserBundle();
    $phone_bundle = $user_wrapper->getPhoneBundle();

    try {
      if ($phone_bundle == 'user') {
        $user_wrapper = new UserWrapper($phone_bundle, (object) $edit);
        $phone = check_plain($user_wrapper->getUserPhone());

        if ($field_subscribe !== 'field_not_required') {
          $subscribe = $user_wrapper->getUserSubscribe();
        }
      }
      elseif ($phone_bundle == 'profile2') {
        $field_phone = variable_get('textmarketer_contacts_telephone', '');

        foreach ($edit as $profile_type => $fields) {
          if (array_key_exists($field_phone, $fields)) {
            $phone = check_plain($fields[$field_phone][LANGUAGE_NONE][0]['value']);
          }
          if (array_key_exists($field_subscribe, $fields)) {
            if ($field_subscribe !== 'field_not_required') {
              $subscribe = $fields[$field_subscribe][LANGUAGE_NONE][0]['value'];
            }
          }
        }
      }
      if ($subscribe || $field_subscribe === 'field_not_required') {

        $this->sendContacts($phone);
      }
    }
    catch (Exception $e) {
      $message = $e->getMessage();
      watchdog('textmarketer_contacts', $message, array(), WATCHDOG_ERROR);
    }
  }

  /**
   * Sends the user contacts to Text Marketer.
   *
   * @param string $numbers
   *   Telephone number.
   *
   * @return array|bool
   *   Calls a request method for sending contacts, returns FALSE on failure.
   */
  protected function sendContacts($numbers) {
    $op = "/services/rest/group/$this->group_id";
    $data = array(
      'numbers'  => $numbers,
      'username' => $this->username,
      'password' => $this->password,
    );

    try {
      return $this->requestAPI($op, $data, 'POST');
    }
    catch (Exception $e) {
      $message = $e->getMessage();
      watchdog('textmareter', 'Error in sendContacts. @message',
        array('@message' => $message), WATCHDOG_ERROR);
      return FALSE;
    }
  }

  /**
   * Imports the telephone numbers from Text Marketer.
   *
   * @return array|bool
   *   Calls a request method for receiving contacts, returns FALSE on failure.
   */
  public function importContacts() {
    $op = "/services/rest/group/$this->group_name";
    $data = array(
      'username' => $this->username,
      'password' => $this->password,
    );
    try {
      return $this->requestAPI($op, $data, 'GET');
    }
    catch (Exception $e) {
      $message = $e->getMessage();
      watchdog('textmareter', 'Error in importContacts. @message',
        array('@message' => $message), WATCHDOG_ERROR);
      return FALSE;
    }
  }

  /**
   * Makes the request to Text Marketer.
   *
   * @param string $op
   *   REST API part of thr URL.
   * @param array $data
   *   The daTa sent to Text Marketer.
   * @param string $method
   *   Determines the request method.
   *
   * @return array
   *   Calls a response parsing method.
   *
   * @throws \Drupal\textmarketer_contacts\Controller\Exception
   *   Error message.
   */
  protected function requestAPI($op, $data = array(), $method = 'GET') {
    $url = $this->base_url . $op;

    if ($method == 'POST') {
      $options = array(
        'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
        'method' => $method,
        'data' => http_build_query($data, '', '&'),
      );
      $response = drupal_http_request($url, $options);
    }
    elseif ($method == 'GET') {
      $response = drupal_http_request($url);

      // Handle response.
      switch ($response->code) {
        case '200':
          return $this->parseResponse($response->data);

        default:
          $message = $response->error . ' ;' . $response->data;
          throw new Exception($message);
      }
    }
  }

  /**
   * Parses the XML response.
   *
   * @param string $response
   *   XML response.
   *
   * @return array
   *   Returns the user results.
   *
   * @throws \Drupal\textmarketer_contacts\Controller\Exception
   *   Throws error message.
   */
  protected function parseResponse($response) {
    $xml = simplexml_load_string($response);
    $json_decode = json_decode(json_encode($xml), TRUE);

    if (!$json_decode) {
      $message = t('Connection failed, check the !settings_page.',
        array(
          '!settings_page' => l(t('settings page'),
            '/admin/config/services/textmarketer-send-contacts'),
        ));
      drupal_set_message($message, 'error');

      throw new Exception($message);
    }
    else {
      $tel_numbers = (isset($json_decode['group']['number'])) ?
        (array) $json_decode['group']['number'] : '';
      $numbers = array();

      if ($tel_numbers) {
        foreach ($tel_numbers as $number) {
          $numbers[] = substr_replace($number, 0, 0, 2);
        }
      }

      return $this->queryDbase($numbers);
    }
  }

  protected function queryDbase($numbers) {
    $field_phone = variable_get('textmarketer_contacts_telephone', '');
    $field_info = field_info_field($field_phone);
    foreach ($field_info['bundles'] as $type => $bundle) {
      $bundle_type = $type;
    }
    $matching_results = $non_matching = array();

    foreach ($numbers as $number) {
      $query = db_select('field_data_' . $field_phone, 't');
      $query->fields('t', array('entity_type', 'entity_id'));
      $query->addField('t', $field_phone . '_value', 'telephone');
      // Check if we have a matching telephone number.
      if ($bundle_type == 'user') {
        $match = $query
          ->condition($field_phone . '_value', db_like($number), 'LIKE')
          ->condition('t.entity_type', $bundle_type, '=')
          ->countQuery()
          ->execute()
          ->fetchField();

        if ($match) {
          // If there is a matching telephone get the rest of the user details.
          $query->Join('users', 'u', 't.entity_id = u.uid');
          $query->fields('u', array('uid', 'name', 'mail', 'created'));
          $query->orderBy('created', 'DESC');
          $matching_results[] = $query->execute()->fetchAll();
        }
        else {
          $non_matching[] = $number;
        }
      }
      elseif ($bundle_type == 'profile2') {
        $match = $query
          ->condition($field_phone . '_value', db_like($number), 'LIKE')
          ->condition('t.entity_type', $bundle_type, '=')
          ->countQuery()
          ->execute()
          ->fetchField();
        if ($match) {
          // If there is a matching telephone get the rest of the user details.
          $query->Join('profile', 'p', 't.entity_id = p.pid');
          $query->Join('users', 'u', 'p.uid = u.uid');
          $query->fields('u', array('uid', 'name', 'mail', 'created'));
          $query->orderBy('created', 'DESC');
          $matching_results[] = $query->execute()->fetchAll();
        }
        else {
          $non_matching[] = $number;
        }
      }
    }

    return [$matching_results, $non_matching];
  }
}
